using HDF5, MAT, CSV, DataFrames

# Read solar & wind data created by the GIS routines in Matlab.
# cf: capacity factor as a function of technology, region (1-6), hour-of-year (1-8760) and resource class (1-5)
# capacitylimits: max potential installed capacity (GW) as a function of technology, region (1-6), hour-of-year (1-8760) and resource class (1-5)

	windvars = matread("GISdata_wind2018_China6.mat")
	solarvars = matread("GISdata_solar2018_China6.mat")
	cf = Dict{Symbol, Array{Float64,3}}()
	capacitylimits = Dict{Symbol, Array{Float64,2}}()
	hoursperyear = 8760

	cf[:wind] = permutedims(windvars["CFtime_windonshoreA"],[2,3,1])
	cf[:offwind] = permutedims(windvars["CFtime_windoffshore"],[2,3,1])
	cf[:pv] = permutedims(solarvars["CFtime_pvplantA"],[2,3,1])
	cf[:roof] = permutedims(solarvars["CFtime_pvrooftop"],[2,3,1])

	for tech in keys(cf)
		cf[tech][isnan.(cf[tech])] .= 0.00
		cf[tech][cf[tech] .< 0.01] .= 0.00		# set small values to 0 for better numerical stability in LP model
	end



    # Capacity limits of renewable technologies

	capacitylimits[:wind] = windvars["capacity_onshoreA"]
	capacitylimits[:offwind] = windvars["capacity_offshore"]
	capacitylimits[:pv] = solarvars["capacity_pvplantA"]
	capacitylimits[:roof] = solarvars["capacity_pvrooftop"]

	Wind=capacitylimits[:wind]*1.25
	offWind=capacitylimits[:offwind]
	Solar=capacitylimits[:pv]*1.2
	Roof=capacitylimits[:roof]


	# Capacity factor of renewable technologies

	cf_Wind=cf[:wind]
	cf_offWind=cf[:offwind]
	cf_Solar=cf[:pv]
	cf_roof=cf[:roof]

	# Demand for China

    demand = CSV.read("demand China.csv";  datarow=2,delim=';')
    DEMAND1=convert(Matrix,demand)

	# Hydro data for China

    storagedam = CSV.read("storage.csv";  datarow=2, delim=';')
    f_inflow = CSV.read("RoRChina.csv";  datarow=2, delim=';')
    damcap = CSV.read("damcap.csv";  datarow=2, delim=';')
    rorcap = CSV.read("rorcap.csv";  datarow=2, delim=';')

    RoR1=convert(Matrix,f_inflow)
    Cap_ror=convert(Matrix,rorcap)
    RoR=randn(8760,6)
    for i in 1:8760
	    for j in 1:6
		    RoR[i,j]=RoR1[i,j]*Cap_ror[j]
	    end
    end

    f_in1=convert(Matrix,f_inflow)
    Cap_dam=convert(Matrix,damcap)
    f_in=randn(8760,6)
    for i in 1:8760
	    for j in 1:6
		    f_in[i,j]=f_in1[i,j]*Cap_dam[j]
	    end
    end

    H_dam0=convert(Matrix,storagedam)*0.6
    H_dammax=convert(Matrix,storagedam)
    H_dammin=convert(Matrix,storagedam)*0.2

	# Investment cost for the generation technologies
	# Investment cost	Variable O&M costs	Fixed O&M costs	Fuel costs	Lifetime Efficiency   Emission from Fuel
	C_g=[ 493   			   3.6   		      12.5			 36            30	     0.35            15             ;
	      800			       3.6				  10.5			 36		       30        0.6             15             ;
	      493   			   3.6   		      12.5			 36            30	     0.35             0             ;
	      800			       3.6				  10.5			 36		       30        0.6              0             ;
	      997 			       0				  33 			 0		       25         1               0             ;
	      2805                 0                  93             0             25         1               0             ;
	      674			       0		 		  8		    	 0		       25         1               0             ;
	      0 			       0				  0			     1000		   25         1               0             ;
	      0 			       0				  0		    	 900		   25         1               0             ;
	      0 			       0				  0		    	 800		   25         1               0             ;
	      0 			       0			      0			     700		   25         1               0             ;
	      0 			       0			      0			     600		   25         1               0             ;
	      2464			       0				  25			 0		       80         1               0             ;
	      3696			       0				  74			 0		       80         1               0             ]


	# Investment cost for transmission and storage
	# Investment cost	Variable O&M costs	Fixed O&M costs		Lifetime      Loss/efficiency
	C_v=[  479 			        0				   9.6			   40         0.016;
	       180                  0                  3.6             40         0.986;
	       156		            0				   0			   10         0.95]


    # Distance between different subregions
	C_tr=[0.0     971.471   880.597  1251.44   1505.2    1219.19;
         971.471     0.0    1459.07   2060.51   2460.83   2125.6;
         880.597  1459.07      0.0     680.953  1347.48   1665.07;
         1251.44   2060.51    680.953     0.0     772.113  1451.63;
         1505.2    2460.83   1347.48    772.113     0.0     990.395;
         1219.19   2125.6    1665.07   1451.63    990.395     0.0]/1000

	# Transmission grids availability
	a_tr=[0  1  1  1  0  1;
          1  0  0  0  0  0;
          1  0  0  1  0  0;
          1  0  1  0  1  1;
          0  0  0  1  0  1;
          1  0  0  1  1  0]
