using HDF5, MAT, CSV, DataFrames

# Read solar & wind data created by the GIS routines in Matlab.
# cf: capacity factor as a function of technology, region (1-13), hour-of-year (1-8760) and resource class (1-5)
# capacitylimits: max potential installed capacity (GW) as a function of technology, region (1-13), hour-of-year (1-8760) and resource class (1-5)

	windvars = matread("GISdata_wind2005_Europe13.mat")
	solarvars = matread("GISdata_solar2005_Europe13.mat")
	cf = Dict{Symbol, Array{Float64,3}}()
	capacitylimits = Dict{Symbol, Array{Float64,2}}()

	hoursperyear = 8760

	cf[:wind] = permutedims(windvars["CFtime_windonshoreA"],[2,3,1])
	cf[:offwind] = permutedims(windvars["CFtime_windoffshore"],[2,3,1])
	cf[:pv] = permutedims(solarvars["CFtime_pvplantA"],[2,3,1])
	cf[:csp] = permutedims(solarvars["CFtime_cspplantA"],[2,3,1])

	for tech in keys(cf)
		cf[tech][isnan.(cf[tech])] .= 0.00
		cf[tech][cf[tech] .< 0.01] .= 0.00		# set small values to 0 for better numerical stability in LP model
	end

	# Capacity limits of renewable technologies

	capacitylimits[:wind] = windvars["capacity_onshoreA"]
	capacitylimits[:offwind] = windvars["capacity_offshore"]
	capacitylimits[:pv] = solarvars["capacity_pvplantA"]
	capacitylimits[:csp] = solarvars["capacity_cspplantA"]

    Wind=capacitylimits[:wind]*1.25
	offWind=capacitylimits[:offwind]
	Solar=capacitylimits[:pv]*1.2

	# Capacity factor of renewable technologies

	cf_Wind=cf[:wind]
	cf_offWind=cf[:offwind]
	cf_Solar=cf[:pv]
	cf_csp=cf[:csp]

    # Demand for Europe

    DemandEU=CSV.read("demand13.csv"; datarow=2, delim=';')
    DEMAND1=convert(Matrix, DemandEU)

	# Hydro data for Europe

    storagedam = CSV.read("EU Hydro 13 storage.csv";  datarow=2, delim=';')
    f_inflow = CSV.read("EU Hydro 13 Res.csv";  datarow=2, delim=';')
    ror = CSV.read("EU Hydro 13 RoR.csv";  datarow=2, delim=';')
    damcap = CSV.read("EU Hydro 13 damcap.csv";  datarow=2, delim=';')
    rorcap = CSV.read("EU Hydro 13 rorcap.csv";  datarow=2, delim=';')

    RoR=convert(Matrix,ror)
    Cap_dam=convert(Matrix,damcap)
    Cap_ror=convert(Matrix,rorcap)
    f_in=convert(Matrix,f_inflow)
    H_dam0=convert(Matrix,storagedam)*1000*0.6
    H_dammax=convert(Matrix,storagedam)*1000
    H_dammin=convert(Matrix,storagedam)*1000*0.2

	# Investment cost for the generation technologies
	# Investment cost	Variable O&M costs	Fixed O&M costs	Fuel costs	Lifetime Efficiency   Emission from Fuel
	C_g=[ 493   			   3.6   		      12.5			 36            30	     0.35            15             ;
	      800			       3.6				  10.5			 36		       30        0.6             15             ;
	      493   			   3.6   		      12.5			 36            30	     0.35             0             ;
	      800			       3.6				  10.5			 36		       30        0.6              0             ;
	      997 			       0				  33 			 0		       25         1               0             ;
	      2805                 0                  93             0             25         1               0             ;
	      674			       0		 		  8		    	 0		       25         1               0             ;
	      0 			       0				  0			     1000		   25         1               0             ;
	      0 			       0				  0		    	 900		   25         1               0             ;
	      0 			       0				  0		    	 800		   25         1               0             ;
	      0 			       0			      0			     700		   25         1               0             ;
	      0 			       0			      0			     600		   25         1               0             ;
	      2464			       0				  25			 0		       80         1               0             ;
	      3696			       0				  74			 0		       80         1               0             ]


	# Investment cost for transmission and storage
	# Investment cost	Variable O&M costs	Fixed O&M costs		Lifetime      Loss/efficiency
	C_v=[  479 			        0				   9.6			   40         0.016;
	       180                  0                  3.6             40         0.986;
	       156		            0				   0			   10         0.95]


	# Distance between different subregions
	C_tr=[0.0     665.137   622.203  1073.15  1437.29   1086.68   1914.02   2084.55   1763.9    1792.57    842.975  3033.16  1146.34;
         665.137     0.0     560.528  1662.46  1180.01   1093.4    1705.6    1618.71   1971.64   2318.26   1445.32   2808.57  1493.3;
         622.203   560.528     0.0    1678.59   816.497   555.276  1307.48   1483.91   1411.69   1840.05   1090.14   2430.91   956.523;
         1073.15   1662.46   1678.59      0.0   2474.14   2040.69   2908.01   3156.02   2434.0    1890.33   1088.81   3997.48  1777.11;
         1437.29   1180.01    816.497  2474.14     0.0     530.887   525.719   798.004  1225.73   2119.65   1689.97   1636.84  1190.87;
         1086.68   1093.4     555.276  2040.69   530.887     0.0     869.414  1328.18    909.081  1619.33   1170.83   1963.62   676.44;
         1914.02   1705.6    1307.48   2908.01   525.719   869.414     0.0     816.661  1137.92   2214.64   2009.71   1123.59  1382.17;
         2084.55   1618.71   1483.91   3156.02   798.004  1328.18    816.661     0.0    1912.81   2902.01   2471.9    1451.0   1985.38;
         1763.9    1971.64   1411.69   2434.0   1225.73    909.081  1137.92   1912.81      0.0    1140.69   1352.66   1829.27   662.973;
         1792.57   2318.26   1840.05   1890.33  2119.65   1619.33   2214.64   2902.01   1140.69      0.0     974.785  2961.22   944.845;
         842.975  1445.32   1090.14   1088.81  1689.97   1170.83   2009.71   2471.9    1352.66    974.785     0.0    3023.99   711.989;
         3033.16   2808.57   2430.91   3997.48  1636.84   1963.62   1123.59   1451.0    1829.27   2961.22   3023.99      0.0   2330.4;
         1146.34   1493.3     956.523  1777.11  1190.87    676.44   1382.17   1985.38    662.973   944.845   711.989  2330.4      0.0]/1000

	# Transmission grids availability
    a_tr=[0 	1 	1 	1 	0 	1 	0 	0 	0 	0 	1 	0 	0;
          1 	0 	1 	1 	1 	1 	0 	1 	0 	0 	0 	0 	0;
          1 	1 	0 	0 	1 	1 	0 	1 	0 	0 	0 	0 	0;
          1 	1 	0 	0 	0 	0 	0 	0 	0 	0 	1 	0 	0;
          0 	1 	1 	0 	0 	1 	1 	1 	0 	0 	0 	0 	0;
          1 	1 	1 	0 	1 	0 	1 	0 	0 	0 	1 	0 	1;
          0 	0 	0 	0 	1 	1 	0 	1 	1 	0 	0 	1 	1;
          0 	1 	1 	0 	1 	0 	1 	0 	0 	0 	0 	0 	0;
          0 	0 	0 	0 	0 	0 	1 	0 	0 	1 	0 	0 	1;
          0 	0 	0 	0 	0 	0 	0 	0 	1 	0 	0 	0 	1;
          1 	0 	0 	1 	0 	1 	0 	0 	0 	0 	0 	0 	1;
          0 	0 	0 	0 	0 	0 	1 	0 	0 	0 	0 	0 	0;
          0 	0 	0 	0 	0 	1 	1 	0 	1 	1 	1 	0 	0]
