using HDF5, MAT, CSV, DataFrames

# Read solar & wind data created by the GIS routines in Matlab.
# cf: capacity factor as a function of technology
# capacitylimits: max potential installed capacity (GW) as a function of technology
	windvars = matread("GISdata_wind2018_Europe13.mat")
	solarvars = matread("GISdata_solar2018_Europe13.mat")
	cf = Dict{Symbol, Array{Float64,3}}()
	capacitylimits = Dict{Symbol, Array{Float64,2}}()
	hoursperyear = 8760
	cf[:wind] = permutedims(windvars["CFtime_windonshoreA"],[2,3,1])
	cf[:offwind] = permutedims(windvars["CFtime_windoffshore"],[2,3,1])
	cf[:pv] = permutedims(solarvars["CFtime_pvplantA"],[2,3,1])
	cf[:csp] = permutedims(solarvars["CFtime_cspplantA"],[2,3,1])

	for tech in keys(cf)
		cf[tech][isnan.(cf[tech])] .= 0.00
		cf[tech][cf[tech] .< 0.01] .= 0.00		# set small values to 0 for better numerical stability in LP model
	end

	# Capacity limits of renewable technologies

	capacitylimits[:wind] = windvars["capacity_onshoreA"]
	capacitylimits[:offwind] = windvars["capacity_offshore"]
	capacitylimits[:pv] = solarvars["capacity_pvplantA"]

	Wind=capacitylimits[:wind]
	offWind=capacitylimits[:offwind]
	Solar=capacitylimits[:pv]


	WIND=[Wind[2,:] Wind[6,:] Wind[12,:]]'*1.25
	OFFWIND=[offWind[2,:] offWind[6,:] offWind[12,:]]'
	SOLAR=[Solar[2,:] Solar[6,:] Solar[12,:]]'*1.2

	# Capacity factor of renewable technologies

	cf_Wind=cf[:wind]
	cf_offWind=cf[:offwind]
	cf_Solar=cf[:pv]

	Cf_Wind=rand(3,5,8760)
	Cf_offWind=rand(3,5,8760)
	Cf_Solar=rand(3,5,8760)

	for i in 1:5
		for j in 1:8760
			Cf_Wind[1,i,j]=cf_Wind[2,i,j]
			Cf_Wind[2,i,j]=cf_Wind[6,i,j]
			Cf_Wind[3,i,j]=cf_Wind[12,i,j]
			Cf_offWind[1,i,j]=cf_offWind[2,i,j]
			Cf_offWind[2,i,j]=cf_offWind[6,i,j]
			Cf_offWind[3,i,j]=cf_offWind[12,i,j]
			Cf_Solar[1,i,j]=cf_Solar[2,i,j]
			Cf_Solar[2,i,j]=cf_Solar[6,i,j]
			Cf_Solar[3,i,j]=cf_Solar[12,i,j]
		end
	end

	# Hydro data for the stylized case
    storagedam = CSV.read("hydrostorage.csv";  datarow=2, delim=';')
    f_inflow = CSV.read("hydrores.csv";  datarow=2, delim=';')
    ror = CSV.read("hydroror.csv";  datarow=2, delim=';')
    damcap = CSV.read("damcap.csv";  datarow=2, delim=';')
    rorcap = CSV.read("rorcap.csv";  datarow=2, delim=';')

    RoR=convert(Matrix,ror)*0.71
    Cap_dam=convert(Matrix,damcap)*0.71
    Cap_ror=convert(Matrix,rorcap)*0.71
    f_in=convert(Matrix,f_inflow)*0.71
    H_dam0=convert(Matrix,storagedam2)*1000*0.6*0.71
    H_dammax=convert(Matrix,storagedam2)*1000*0.71
    H_dammin=convert(Matrix,storagedam2)*1000*0.2*0.71

	# Investment cost for the generation technologies
	# Investment cost	Variable O&M costs	Fixed O&M costs	Fuel costs	Lifetime Efficiency   Emission from Fuel
	C_g=[ 493   			   3.6   		      12.5			 36            30	     0.35            15             ;
	      800			       3.6				  10.5			 36		       30        0.6             15             ;
	      493   			   3.6   		      12.5			 36            30	     0.35             0             ;
	      800			       3.6				  10.5			 36		       30        0.6              0             ;
	      997 			       0				  33 			 0		       25         1               0             ;
	      2805                 0                  93             0             25         1               0             ;
	      674			       0		 		  8		    	 0		       25         1               0             ;
	      0 			       0				  0			     1000		   25         1               0             ;
	      0 			       0				  0		    	 900		   25         1               0             ;
	      0 			       0				  0		    	 800		   25         1               0             ;
	      0 			       0			      0			     700		   25         1               0             ;
	      0 			       0			      0			     600		   25         1               0             ;
	      2464			       0				  25			 0		       80         1               0             ;
	      3696			       0				  74			 0		       80         1               0             ]


	# Investment cost for transmission and storage
	# Investment cost	Variable O&M costs	Fixed O&M costs		Lifetime      Loss/efficiency
	C_v=[  479 			        0				   9.6			   40         0.016;
	  	   180                  0                  3.6             40         0.986;
	  	   156		            0				   0			   10         0.95]


	# Distance between different subregions
    C_tr=[0	 1200 	2400;
		  1200 	0 	1600;
		  2400 	1600 	0]/1000
