using HDF5, MAT

# Read solar & wind data created by the GIS routines in Matlab.
# cf: capacity factor as a function of technology, region (1-17), hour-of-year (1-8760) and resource class (1-5)
# capacitylimits: max potential installed capacity (GW) as a function of technology, region (1-17), hour-of-year (1-8760) and resource class (1-5)
function readsolarwind_Matlab()
	# do something with B classes (and pvrooftop) later
	windvars = matread("GISdata_wind2016_europe17.mat")
	solarvars = matread("GISdata_solar2016_europe17.mat")

	cf = Dict{Symbol, Array{Float64,3}}()
	capacitylimits = Dict{Symbol, Array{Float64,2}}()

	hoursperyear = 8760
	# Important! Sync wind & solar time series with demand.
	# First wind data is at 00:00 and first solar data is at 07:00, so need to shift them to make them simultaneous.
	# Let's assume first demand data is at 00:00 (not 100% sure about this).
	cf[:wind] = permutedims(windvars["CFtime_windonshoreA"][25:hoursperyear+24,:,:], [2,3,1])
	cf[:offwind] = permutedims(windvars["CFtime_windoffshore"][25:hoursperyear+24,:,:], [2,3,1])
	cf[:pv] = permutedims(solarvars["CFtime_pvplantA"][18:hoursperyear+17,:,:], [2,3,1])
	cf[:csp] = permutedims(solarvars["CFtime_cspplantA"][18:hoursperyear+17,:,:], [2,3,1])

	for tech in keys(cf)
		cf[tech][isnan.(cf[tech])] .= 0.00
		cf[tech][cf[tech] .< 0.01] .= 0.00		# set small values to 0 for better numerical stability in LP model
	end

	capacitylimits[:wind] = windvars["capacity_onshoreA"]
	capacitylimits[:offwind] = windvars["capacity_offshore"]
	capacitylimits[:pv] = solarvars["capacity_pvplantA"]
	capacitylimits[:csp] = solarvars["capacity_cspplantA"]

	# Capacity limits of renewable technologies

	Wind=capacitylimits[:wind]
	offWind=capacitylimits[:offwind]
	Solar=capacitylimits[:pv]

	# Capacity factor of renewable technologies

	cf_Wind=cf[:wind]
	cf_offWind=cf[:offwind]
	cf_Solar=cf[:pv]
	cf_csp=cf[:csp]

	return Wind, offWind, Solar, cf_Wind, cf_offWind, cf_Solar, cf_csp
end

Wind, offWind, Solar, cf_Wind, cf_offWind, cf_Solar, cf_csp = readsolarwind_Matlab()
