using JuMP, Gurobi, Plots, DataFrames
include("Data input.jl")

Demand_no = CSV.read("Demand North.csv";  datarow=2,delim=';')
Demand_No=convert(Array{Float64,2},Demand_no)
Demand_so = CSV.read("Demand South.csv";  datarow=2,delim=';')
Demand_So=convert(Array{Float64,2},Demand_sp)
Demand_ce = CSV.read("Demand Central.csv";  datarow=2,delim=';')
Demand_Ce=convert(Array{Float64,2},Demand_de)

DEMAND1=[Demand_No[:,1] Demand_Ce[:,1] Demand_So[:,1]]

# Number of thermal technologies
m=4
# Number of variable renewable technologies
n=3
# Total number of generation technologies except hydropower
N=12
# Number of regions
R=3

# Number of technology class
Class=5
# Number of time slices
T=8760
#Interest rate
η=0.05
#Transmission Price
p_tr=0.000000001


# The model
Inva = Model(solver=GurobiSolver(Method=2,Crossover=0))
# The variables
#Capacity of thermal generation technology n in region r [MW]
@variable(Inva, x[1:m,1:R]>=0)
# Capacity of renewable generation technology n in region r [MW]
@variable(Inva, y[1:n,1:R,1:Class]>=0)
#Capacity of transmission between region r and r' [MW]
@variable(Inva, z[1:R,1:R] >=0)
#Storage in region r [MWh]
@variable(Inva, u[1:R] >=0)
#Generation using technology n in region r during time-step t [MWh]
@variable(Inva, g[1:N,1:R,1:T] >=0)
#Electricity transacted between region r and r' during time-step t [MWh]
@variable(Inva, γ[1:R,1:R,1:T] >=0)
#Electricity into storage in region r during time-step t [MWh]
@variable(Inva, θ_in[1:R,1:T] >=0)
#Electricity out of storage in region r during time-step t [MWh]
@variable(Inva, θ_out[1:R,1:T] >=0)
#Storage level in region r during time-step t [MWh]
@variable(Inva, θ[1:R,1:T] >=0)
#Run of river hydropower production in region r during time-step t [MWh]
@variable(Inva, h_RoR[1:R,1:T] >=0)
#Bypass outflow from reservior in region r during time-step t[MWh]
@variable(Inva, f_outby[1:R,1:T] >=0)
#Outflow from reservior in region r during time-step t [MWh]
@variable(Inva, f_out[1:R,1:T] >=0)
#Total outflow from reservior in region r during time-step t [MWh]
@variable(Inva, f_out0[1:R,1:T] >=0)
#Reservoir storage in region r during time-step t [MWh]
@variable(Inva, H_dam[1:R,1:T] >=0)
#Curtailment in region r during time-step t [MWh]
@variable(Inva, Ct[1:R,1:T] >=0)

@objective(Inva, Min, (1000000*sum((C_g[i,1]/((1-1/(1+η)^C_g[i,5])/η)+C_g[i,3])*x[i,j] for i in 1:m for j in 1:R) +
                        1000000*sum((C_g[(i+m),1]/((1-1/(1+η)^C_g[(i+m),5])/η)+C_g[(i+m),3])*y[i,j,l] for i in 1:n for j in 1:R for l in 1:Class) +
                        1000*sum((C_g[i,2]+C_g[i,4]/C_g[i,6])*g[i,j,k] for i in 1:N for j in 1:R for k in 1:T)+
                        1000000*(C_g[(1+N),1]/((1-1/(1+η)^C_g[(1+N),5])/η)+C_g[(1+N),3])*sum(Cap_dam)+1000000*(C_g[(2+N),1]/((1-1/(1+η)^C_g[(2+N),5])/η)+C_g[(2+N),3])*sum(Cap_ror)+
                        1000000*0.5*sum(((C_tr[i,j]*C_v[1,1]+C_v[2,1])/((1-1/(1+η)^C_v[1,4])/η)+C_tr[i,j]*C_v[1,3]+C_v[2,3])*z[i,j] for i in 1:R for j in 1:R)+
                        1000*sum(p_tr*γ[i,j,k] for i in 1:R for j in 1:R for k in 1:T)+
                        1000000*sum((C_v[3,1]/((1-1/(1+η)^C_v[3,4])/η))*u[i] for i in 1:R)))


@constraints Inva begin

Thermal_generation1[i in 1:m, j in 1:R, k in 1:T],
g[i,j,k] <= x[i,j]*1

Thermal_generation2[j in 1:R],
sum((g[i,j,k]/C_g[i,6]) for i in 3:4 for k in 1:T) <=sum(DEMAND1[k,j] for k in 1:T)*0.05

#Constraints for wind and solar

Wind_capacity[j in 1:R, l in 1:Class],
y[1,j,l] <= WIND[j,l]

offWind_capacity[j in 1:R, l in 1:Class],
y[2,j,l] <= OFFWIND[j,l]

Solar_capacity[j in 1:R, l in 1:Class],
y[3,j,l] <= SOLAR[j,l]

Wind_generation1[ j in 1:R, k in 1:T],
g[5,j,k] <= sum(y[1,j,l]*Cf_Wind[j,l,k] for l in 1:Class)

offWind_generation1[ j in 1:R, k in 1:T],
g[6,j,k] <= sum(y[2,j,l]*Cf_offWind[j,l,k] for l in 1:Class)

Solar_generation1[ j in 1:R, k in 1:T],
g[7,j,k] <= sum(y[3,j,l]*Cf_Solar[j,l,k] for l in 1:Class)


#Constraints for demand-response

Demand_curtailment1[j in 1:R, k in 1:T],
g[8,j,k] <=DEMAND1[k,j]*0.01

Demand_curtailment2[j in 1:R, k in 1:T],
g[9,j,k] <=DEMAND1[k,j]*0.01

Demand_curtailment3[j in 1:R, k in 1:T],
g[10,j,k] <=DEMAND1[k,j]*0.01

Demand_curtailment4[j in 1:R, k in 1:T],
g[11,j,k] <=DEMAND1[k,j]*0.01

Demand_curtailment5[j in 1:R, k in 1:T],
g[12,j,k] <=DEMAND1[k,j]*0.01

#Constraints for transmission

Transmission1[i in 1:R,j in 1:R],
z[i,i] == 0

Transmission2[i in 1:R,j in 1:R],
z[i,j] == z[j,i]

Transmission3[i in 1:R, j in 1:R, k in 1:T],
γ[i,j,k] <= z[i,j]*1

#Constraints for storage

Storage1[i in 1:R, j in 1:T],
θ[i,j] <= u[i]

Storage2[i in 1:R, j in 1:T],
θ_out[i,j]/C_v[3,5] <= θ[i,j]

Storage3[i in 1:R, j in 1:T],
θ_in[i,j]*C_v[3,5] <= u[i]-θ[i,j]

Storage4[i in 1:R, j in 2:T],
θ[i,j] == θ[i,(j-1)] + C_v[3,5]*θ_in[i,j] - θ_out[i,j]/C_v[3,5]

Storage5[i in 1:R, j in 2:T],
θ[i,1] == 0

#Constraints for hydropower

Reservior1[i in 1:R, j in 1:T],
f_out[i,j] <= Cap_dam[i]*1

Reservior2[i in 1:R, j in 1:T],
f_out0[i,j] == f_out[i,j]+f_outby[i,j]

Reservior3[i in 1:R,j in 1:T],
f_out0[i,j] >= sum(f_in[:,i])/365/24*0.05

Reservior4[i in 1:R, j in 2:T],
H_dam[i,j] == H_dam[i,(j-1)]+f_in[(cld(j,24)),i]/24-f_out0[i,j]

Reservior5[i in 1:R],
H_dam[i,1] == H_dam0[1,i]

Reservior6[i in 1:R],
H_dam[i,T] == H_dam0[1,i]

Reservior7[i in 1:R, j in 1:T],
H_dam[i,j] <= H_dammax[1,i]

Reservior8[i in 1:R, j in 1:T],
H_dammin[1,i] <= H_dam[i,j]

Runofriver1[i in 1:R, j in 1:T],
h_RoR[i,j] <= RoR[(cld(j,24)),i]/24

Runofriver2[i in 1:R, j in 1:T],
h_RoR[i,j] <= Cap_ror[1,i]

#Constraints for CO2 emission

Carbon_emmision1,
1000*sum(C_g[i,7]/C_g[i,6]*44/12*3600*g[i,j,k] for i in 1:m for j in 1:R for k in 1:T) - sum(DEMAND1[k,j] for k in 1:T for j in 1:R)*10000000<= 0

#Constraints for demand

Demand[j in 1:R, k in 1:T],
sum(g[i,j,k] for i in 1:N)+(sum(γ[i,j,k]*(1-C_v[1,5]*C_tr[i,j])*C_v[2,5] for i in 1:R))-sum(γ[j,i,k] for i in 1:R)+θ_out[j,k]-θ_in[j,k]+h_RoR[j,k]+f_out[j,k]-Ct[j,k] == DEMAND1[k,j]

end

statusInva = solve(Inva)
print(statusInva)


#function PrintSolution
if statusInva == :Optimal
print("Optimal objective value = ", getobjectivevalue(Inva),
"\nOptimal x = ", getvalue(x),
"\nOptimal y = ", getvalue(y),
"\nOptimal z = ", getvalue(z),
"\nOptimal u = ", getvalue(u),
"\nOptimal z_total = ", sum(getvalue(z)),
"\nOptimal g_OCGT = ", sum(getvalue(g[1,:,:])),
"\nOptimal g_CCGT = ", sum(getvalue(g[2,:,:])),
"\nOptimal g_BOCGT = ", sum(getvalue(g[3,:,:])),
"\nOptimal g_BCCGT = ", sum(getvalue(g[4,:,:])),
"\nOptimal g_Wind = ", sum(getvalue(g[5,:,:])),
"\nOptimal g_OffWind = ", sum(getvalue(g[6,:,:])),
"\nOptimal g_Solar = ", sum(getvalue(g[7,:,:])),
"\nOptimal g_D1 = ", sum(getvalue(g[8,:,:])),
"\nOptimal g_D2 = ", sum(getvalue(g[9,:,:])),
"\nOptimal g_D3 = ", sum(getvalue(g[10,:,:])),
"\nOptimal g_D4 = ", sum(getvalue(g[11,:,:])),
"\nOptimal g_D5 = ", sum(getvalue(g[12,:,:])),
"\nOptimal Trade = ", sum(getvalue(γ)),
"\nOptimal θ_in = ", sum(getvalue(θ_in)),
"\nOptimal θ_out = ", sum(getvalue(θ_out)),
"\nOptimal g_Hydro = ", sum(getvalue(f_out)),
"\nOptimal Curtailment = ", sum(getvalue(Ct)),
"\nDemand = ", sum(DEMAND1))
else
    error("No solution")
end

Cap_Hydro=sum(Cap_dam[1,:]+Cap_ror[1,:])
Cap_OCGT=sum(getvalue(x[1,:]))
Cap_CCGT=sum(getvalue(x[2,:]))
Cap_BOCGT=sum(getvalue(x[3,:]))
Cap_BCCGT=sum(getvalue(x[4,:]))
Cap_Wind=sum(getvalue(y[1,:,:]))
Cap_offWind=sum(getvalue(y[2,:,:]))
Cap_Solar=sum(getvalue(y[3,:,:]))
Cap_Trans=sum(getvalue(z))
Cap_Storage=sum(getvalue(u))
Gen_OCGT=sum(getvalue(g[1,:,:]))
Gen_CCGT=sum(getvalue(g[2,:,:]))
Gen_BOCGT=sum(getvalue(g[3,:,:]))
Gen_BCCGT=sum(getvalue(g[4,:,:]))
Gen_Wind=sum(getvalue(g[5,:,:]))
Gen_offWind=sum(getvalue(g[6,:,:]))
Gen_Solar=sum(getvalue(g[7,:,:]))
Gen_D1=sum(getvalue(g[8,:,:]))
Gen_D2=sum(getvalue(g[9,:,:]))
Gen_D3=sum(getvalue(g[10,:,:]))
Gen_D4=sum(getvalue(g[11,:,:]))
Gen_D5=sum(getvalue(g[12,:,:]))
Trade_in=sum(getvalue(γ[:,:,:]))
Trade_out=sum(getvalue(γ[:,:,:]))
Storage_in=sum(getvalue(θ_in[:,:]))
Storage_out=sum(getvalue(θ_out[:,:]))
Gen_RoR=sum(getvalue(h_RoR))
Dam_out=sum(getvalue(f_out))
Curtailment =sum(getvalue(Ct))
Obj=getobjectivevalue(Inva)
Total_demand=sum(DEMAND1[:,:])
Cost=Obj/Total_demand
Cal=[Cap_Hydro;Cap_OCGT;Cap_CCGT;Cap_BOCGT;Cap_BCCGT;Cap_Wind;Cap_offWind;Cap_Solar;Cap_Trans;Cap_Storage;Gen_RoR;Dam_out;
Gen_OCGT;Gen_CCGT;Gen_BOCGT;Gen_BCCGT;Gen_Wind;Gen_offWind;Gen_Solar;Gen_D1;Gen_D2;Gen_D3;Gen_D4;Gen_D5;Trade_in;Trade_out;
Storage_in;Storage_out;Curtailment;Obj;Total_demand;Cost]

Calculation=convert(DataFrame, Cal')
CSV.write("CNNN.csv",Calculation)
